﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.SimulationCore {
  [StorableClass]
  public abstract class Action<TModel> : IAction<TModel> where TModel : IModel {
    public IActivity<TModel> Mandate { get; protected set; }

    [Storable]
    private IActivity<TModel> StorableMandate {
      get { return Mandate; }
      set { Mandate = value; }
    }

    [StorableConstructor]
    protected Action(bool deserializing) { }
    protected Action(Action<TModel> original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
      Mandate = cloner.Clone(original.Mandate);
    }
    protected Action() { }
    protected Action(IActivity<TModel> mandate) {
      Mandate = mandate;
    }

    public abstract void Execute(TModel model);

    public object Clone() { return Clone(new Cloner()); }
    public abstract IDeepCloneable Clone(Cloner cloner);
  }
}
